//***********************************************************************
// TMP102 Breakout Board test code
// SparkFun Electronoics 08/24/09
// Written for ATMega128
//***********************************************************************

#include <stdio.h>
#include <avr/io.h>
#include <avr/sfr_defs.h>
#include "i2c.h"

#define FOSC 8000000 //1MHz internal osc
#define BAUD 9600
#define MYUBRR (((((FOSC * 10) / (16L * BAUD)) + 5) / 10) - 1)

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

// TMP102 Variables
#define RD_ADDR 	0x91
#define WR_ADDR 	0x90
#define TEMP_REG 	0x00

static int ioinit(void);
static int uart_putchar(char c, FILE *stream);
void delay_ms(uint16_t x);
void delay_us(uint16_t x);
void delay(uint16_t ms, uint16_t tenth_ms);

// I2C Functions
unsigned char i2cGetReceivedByte(void);
void i2cReceiveByte(unsigned char ackFlag);
void i2cSendByte(unsigned char data);
void i2cWaitForComplete(void);
void i2cSendStop(void);
void i2cSendStart(void);
void send_byte(uint8_t byte);

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

int main(void)
{
	ioinit();
	
	printf("\n\rTMP102 Test Routine\n\r");
	delay_ms(100);

	uint8_t in_byte, i, msb, lsb;
	uint8_t temp = 0;
	
	uint8_t atemp[2];
	
	TWCR = 0x00;
	//initilize I2C hardware
	TWBR = 32;

	//Send start condition 
	i2cSendStop();
	i2cSendStart();	
    i2cWaitForComplete();
	
	//send write
	i2cSendByte(WR_ADDR);
	TWCR = (1<<TWINT)|(1<<TWEN); //send	
	i2cWaitForComplete();
	
	//write pointer register
	i2cSendByte(TEMP_REG);
	TWCR = (1<<TWINT)|(1<<TWEN);
	i2cWaitForComplete();
	i2cSendStop();
	
	while(1)
	{
		i2cSendStart();	//Repeat Start
		i2cWaitForComplete();
		i2cSendByte(RD_ADDR); //Ask the IC to report on the last command
		TWCR = (1<<TWINT)|(1<<TWEN); //send	
		delay_ms(10);
		
		if(inb(TWSR) == TW_MR_SLA_ACK)
		{
	
			for(i=0; i<2; i++)
			{
				i2cReceiveByte(TRUE);
				i2cWaitForComplete();
				in_byte = i2cGetReceivedByte();
				atemp[i] = in_byte;
			}
	
			msb = atemp[0];
			lsb = atemp[1];
			
			temp = msb<<8;
			temp = temp|lsb;
		
		}
	
		printf("%d\n\r",temp);
		delay_ms(5000);
	}
	
	
	//send stop condition
	i2cSendStop();
	
}

static int ioinit(void)
{
	//1 = output, 0 = input
    DDRB = 0b11100111; //PB4 = MISO 
    DDRC = 0b00110001; //Output on PORTC0, PORTC4 (SDA), PORTC5 (SCL), all others are inputs
    DDRD = 0b11110110; //PORTD (RX on PD0), input on PD2
	
	UBRR0H = MYUBRR >> 8;
    UBRR0L = MYUBRR;
    UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	stdout = &mystdout; //Required for printf init
	return 0;
}


static int uart_putchar(char c, FILE *stream)
{
    if (c == '\n') uart_putchar('\r', stream);
  
    loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = c;
    
    return 0;
}


void delay(uint16_t ms, uint16_t tenth_ms)
{
	delay_ms(ms);
	
	for(; tenth_ms>0 ; tenth_ms--){ delay_us(12); }
}

void send_byte(uint8_t byte)
{
	i2cSendByte(byte);
	
	TWCR = (1<<TWINT)|(1<<TWEN); //send	
	
	i2cWaitForComplete();
}

//General short delays
void delay_ms(uint16_t x)
{
	for (; x > 0 ; x--)
	{
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
		delay_us(12);
	}
}


void delay_us(uint16_t x)
{
	for(uint16_t i = 0; i<x; i++){}
}
////////////////////////////////////////////////////////////////////////////////
///==============I2C Functions=======================================///////////
////////////////////////////////////////////////////////////////////////////////

void i2cSendStart(void)
{
	//WRITE_sda();
	// send start condition
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
}

void i2cSendStop(void)
{
	// transmit stop condition
        TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
}

void i2cWaitForComplete(void)
{
	// wait for i2c interface to complete operation
        while (!(TWCR & (1<<TWINT)));
}

void i2cSendByte(unsigned char data)
{

	//WRITE_sda();
	// save data to the TWDR
	TWDR = data;
	// begin send
	//TWCR = (1<<TWINT)|(1<<TWEN);
}

void i2cReceiveByte(unsigned char ackFlag)
{
	// begin receive over i2c
	if( ackFlag )
	{
		// ackFlag = TRUE: ACK the recevied data
		outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
	}
	else
	{
		// ackFlag = FALSE: NACK the recevied data
		outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
	}
}

unsigned char i2cGetReceivedByte(void)
{
	// retieve received data byte from i2c TWDR
	return( inb(TWDR) );
}
